@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2015 Thomas Perl <thp.io/about>
+# Copyright (c) 2008-2016 Thomas Perl <thp.io/about>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -29,8 +29,15 @@
 
 
 import re
+import os
+import subprocess
+import logging
+
+logger = logging.getLogger(__name__)
+
+
+def html2text(data, method='lynx'):
 
-def html2text(data, method='lynx', utf8=False):
     """
     Convert a string consisting of HTML to plain text
     for easy difference checking.
@@ -39,39 +46,32 @@ def html2text(data, method='lynx', utf8=False):
      'lynx' (default) - Use "lynx -dump" for conversion
      'html2text'      - Use "html2text -nobs" for conversion
      're'             - A simple regex-based HTML tag stripper
-
-    If utf8 is True, the data will be handled as utf-8 by Lynx and
-    html2text (if possible). It seems like only the Debian-provided
-    version of html2text has support for the "-utf8" command line
-    flag, so this might not work on non-Debian systems.
-
-    Dependencies: apt-get install lynx html2text
     """
-    if isinstance(data, unicode):
-        data = data.encode('utf-8')
-
     if method == 're':
         stripped_tags = re.sub(r'<[^>]*>', '', data)
         d = '\n'.join((l.rstrip() for l in stripped_tags.splitlines() if l.strip() != ''))
         return d
 
     if method == 'lynx':
-        cmd = ['lynx', '-dump', '-stdin']
-
-        if utf8:
-            cmd.append('-assume_charset=UTF-8')
+        cmd = ['lynx', '-dump', '-stdin', '-assume_charset=UTF-8']
+        # For some reason it looks like lynx always(?) outputs Latin-1
+        stdout_encoding = 'latin-1'
     elif method == 'html2text':
-        cmd = ['html2text', '-nobs']
-
-        if utf8:
-            cmd.append('-utf8')
+        cmd = ['html2text', '-nobs', '-utf8']
+        stdout_encoding = 'utf-8'
     else:
-        return data
+        raise ValueError('Unknown html2text method: %r' % (method,))
 
-    import subprocess
-    html2text = subprocess.Popen(cmd, stdin=subprocess.PIPE, \
-            stdout=subprocess.PIPE)
-    (stdout, stderr) = html2text.communicate(data)
+    logger.debug('Command: %r, stdout encoding: %s', cmd, stdout_encoding)
+
+    env = {}
+    env.update(os.environ)
+    env['LANG'] = 'en_US.utf-8'
+    env['LC_ALL'] = 'en_US.utf-8'
+
+    html2text = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
+    stdout, stderr = html2text.communicate(data.encode('utf-8'))
+    stdout = stdout.decode(stdout_encoding)
 
     if method == 'lynx':
         # Lynx translates relative links in the mode we use it to:
@@ -79,25 +79,14 @@ def html2text(data, method='lynx', utf8=False):
 
         # Recent versions of lynx (seen in 2.8.8pre1-1) do not include the
         # "localhost" in the file:// URLs; see Debian bug 732112
-        stdout = re.sub(r'file:///tmp/[^/]*/', '', stdout)
+        stdout = re.sub(r'file://%s/[^/]*/' % (os.environ.get('TMPDIR', '/tmp'),), '', stdout)
 
         # Use the following regular expression to remove the unnecessary
         # parts, so that [RANDOM STRING] (changing on each call) does not
         # expose itself as change on the website (it's a Lynx-related thing
         # Thanks to Evert Meulie for pointing that out
-        stdout = re.sub(r'file://localhost/tmp/[^/]*/', '', stdout)
+        stdout = re.sub(r'file://localhost%s/[^/]*/' % (os.environ.get('TMPDIR', '/tmp'),), '', stdout)
         # Also remove file names like L9816-5928TMP.html
         stdout = re.sub(r'L\d+-\d+TMP.html', '', stdout)
 
-    return stdout
-
-
-if __name__ == '__main__':
-    import sys
-
-    if len(sys.argv) == 2:
-        print html2text(open(sys.argv[1]).read())
-    else:
-        print 'Usage: %s document.html' % (sys.argv[0])
-        sys.exit(1)
-
+    return stdout.strip()
